import {
  CommandInteraction,
  Constants,
  InteractionDataOptionsString,
  TextableChannel
} from "@projectdysnomia/dysnomia";
import Command from "./command";
import { findTimeZone } from "timezone-support";

import { getTimeFor, updateTimeFor } from "../utils/time";
import { Optional, unravelOptionalOption } from "../utils/options";

async function failMessage(interaction: CommandInteraction<TextableChannel>) {
  await interaction.createMessage({
    content:
      ":warning: I didn't understand that time zone. Try an [IANA time zone code](https://en.wikipedia.org/wiki/List_of_tz_database_time_zones).",
    flags: Constants.MessageFlags.EPHEMERAL
  });
}

const timezone: Command = {
  name: "timezone",
  description: "Set your time zone.",
  options: [
    {
      name: "tz",
      description:
        "The time zone from the IANA time zone database - e.g. 'America/New_York'.",
      type: Constants.ApplicationCommandOptionTypes.STRING
    }
  ],
  command: async (interaction) => {
    const options = interaction.data.options ?? [];
    const tzOption = unravelOptionalOption<
      Optional<InteractionDataOptionsString>
    >("tz", options);

    const user = (interaction.member || interaction.user)!.id;

    if (tzOption == null) {
      const worked = await updateTimeFor(user, null);
      if (!worked) {
        await failMessage(interaction);
        return;
      }

      await interaction.createMessage({
        content: ":white_check_mark: Unset your time zone.",
        flags: Constants.MessageFlags.EPHEMERAL
      });

      return;
    }

    let tz: string;
    try {
      tz = findTimeZone(tzOption.value).name;
    } catch (err) {
      await failMessage(interaction);
      return;
    }

    const worked = await updateTimeFor(user, tz);
    if (!worked) {
      await failMessage(interaction);
      return;
    }

    const time = await getTimeFor(user);
    if (time == null) {
      await failMessage(interaction);
      return;
    }

    const hour = time.getHours();
    const minutes = time.getMinutes();

    const amPM = hour >= 12 ? "PM" : "AM";

    const hourStr = hour % 12 || 12;
    const minutesStr = minutes.toString().padStart(2, "0");
    const timeStr = `${hourStr}:${minutesStr} ${amPM}`;

    let greetingStr = "morning";
    if (hour >= 12 && hour < 18) {
      greetingStr = "afternoon";
    } else if (hour >= 18 && hour < 22) {
      greetingStr = "evening";
    } else if (hour >= 22 || hour < 4) {
      greetingStr = "night";
    }

    await interaction.createMessage({
      content: `:white_check_mark: Good ${greetingStr}! It's ${timeStr} for you.`,
      flags: Constants.MessageFlags.EPHEMERAL
    });
  }
};

export default timezone;
